@@ -348,7 +348,7 @@ static bool create_latch(RTLIL::Module *module, const LibertyAst *node, bool fla
 		RTLIL::Cell *enable_gate = module->addCell(NEW_ID, enable_polarity ? ID($_OR_) : ID($_AND_));
 		enable_gate->setPort(ID::A, enable_sig);
 		enable_gate->setPort(ID::B, clear_enable);
-		enable_gate->setPort(ID::Y, data_sig = module->addWire(NEW_ID));
+		enable_gate->setPort(ID::Y, enable_sig = module->addWire(NEW_ID));
 	}
 
 	if (preset_sig.size() == 1)
@@ -376,7 +376,7 @@ static bool create_latch(RTLIL::Module *module, const LibertyAst *node, bool fla
 		RTLIL::Cell *enable_gate = module->addCell(NEW_ID, enable_polarity ? ID($_OR_) : ID($_AND_));
 		enable_gate->setPort(ID::A, enable_sig);
 		enable_gate->setPort(ID::B, preset_enable);
-		enable_gate->setPort(ID::Y, data_sig = module->addWire(NEW_ID));
+		enable_gate->setPort(ID::Y, enable_sig = module->addWire(NEW_ID));
 	}
 
 	cell = module->addCell(NEW_ID, stringf("$_DLATCH_%c_", enable_polarity ? 'P' : 'N'));
@@ -462,6 +462,9 @@ struct LibertyFrontend : public Frontend {
 		log("    -ignore_miss_data_latch\n");
 		log("        ignore latches with missing data and/or enable pins\n");
 		log("\n");
+		log("    -ignore_buses\n");
+		log("        ignore cells with bus interfaces (wide ports)\n");
+		log("\n");
 		log("    -setattr <attribute_name>\n");
 		log("        set the specified attribute (to the value 1) on all loaded modules\n");
 		log("\n");
@@ -478,6 +481,7 @@ struct LibertyFrontend : public Frontend {
 		bool flag_ignore_miss_func = false;
 		bool flag_ignore_miss_dir  = false;
 		bool flag_ignore_miss_data_latch = false;
+		bool flag_ignore_buses = false;
 		bool flag_unit_delay = false;
 		std::vector<std::string> attributes;
 
@@ -514,6 +518,10 @@ struct LibertyFrontend : public Frontend {
 				flag_ignore_miss_data_latch = true;
 				continue;
 			}
+			if (arg == "-ignore_buses") {
+				flag_ignore_buses = true;
+				continue;
+			}
 			if (arg == "-setattr" && argidx+1 < args.size()) {
 				attributes.push_back(RTLIL::escape_id(args[++argidx]));
 				continue;
@@ -531,7 +539,7 @@ struct LibertyFrontend : public Frontend {
 
 		log_header(design, "Executing Liberty frontend: %s\n", filename.c_str());
 
-		LibertyParser parser(*f);
+		LibertyParser parser(*f, filename);
 		int cell_count = 0;
 
 		std::map<std::string, std::tuple<int, int, bool>> global_type_map;
@@ -542,27 +550,13 @@ struct LibertyFrontend : public Frontend {
 			if (cell->id != "cell" || cell->args.size() != 1)
 				continue;
 
-			std::string cell_name = RTLIL::escape_id(cell->args.at(0));
-
-			if (design->has(cell_name)) {
-				Module *existing_mod = design->module(cell_name);
-				if (!flag_nooverwrite && !flag_overwrite && !existing_mod->get_bool_attribute(ID::blackbox)) {
-					log_error("Re-definition of cell/module %s!\n", log_id(cell_name));
-				} else if (flag_nooverwrite) {
-					log("Ignoring re-definition of module %s.\n", log_id(cell_name));
-					continue;
-				} else {
-					log("Replacing existing%s module %s.\n", existing_mod->get_bool_attribute(ID::blackbox) ? " blackbox" : "", log_id(cell_name));
-					design->remove(existing_mod);
-				}
-			}
-
 			// log("Processing cell type %s.\n", RTLIL::unescape_id(cell_name).c_str());
 
 			std::map<std::string, std::tuple<int, int, bool>> type_map = global_type_map;
 			parse_type_map(type_map, cell);
 
 			RTLIL::Module *module = new RTLIL::Module;
+			std::string cell_name = RTLIL::escape_id(cell->args.at(0));
 			module->name = cell_name;
 
 			if (flag_lib)
@@ -571,9 +565,15 @@ struct LibertyFrontend : public Frontend {
 			if (flag_wb)
 				module->set_bool_attribute(ID::whitebox);
 
+			const LibertyAst *area = cell->find("area");
+			if (area)
+				module->attributes[ID::area] = area->value;
+
 			for (auto &attr : attributes)
 				module->attributes[attr] = 1;
 
+			bool simple_comb_cell = true, has_outputs = false;
+
 			for (auto node : cell->children)
 			{
 				if (node->id == "pin" && node->args.size() == 1) {
@@ -595,6 +595,12 @@ struct LibertyFrontend : public Frontend {
 
 				if (node->id == "bus" && node->args.size() == 1)
 				{
+					if (flag_ignore_buses) {
+						log("Ignoring cell %s with a bus interface %s.\n", log_id(module->name), node->args.at(0).c_str());
+						delete module;
+						goto skip_cell;
+					}
+
 					if (!flag_lib)
 						log_error("Error in cell %s: bus interfaces are only supported in -lib mode.\n", log_id(cell_name));
 
@@ -609,6 +615,8 @@ struct LibertyFrontend : public Frontend {
 					if (!dir || (dir->value != "input" && dir->value != "output" && dir->value != "inout" && dir->value != "internal"))
 						log_error("Missing or invalid direction for bus %s on cell %s.\n", node->args.at(0).c_str(), log_id(module->name));
 
+					simple_comb_cell = false;
+
 					if (dir->value == "internal")
 						continue;
 
@@ -656,12 +664,19 @@ struct LibertyFrontend : public Frontend {
 				{
 					const LibertyAst *dir = node->find("direction");
 
+					if (dir->value == "internal" || dir->value == "inout")
+						simple_comb_cell = false;
+
 					if (flag_lib && dir->value == "internal")
 						continue;
 
 					RTLIL::Wire *wire = module->wires_.at(RTLIL::escape_id(node->args.at(0)));
 					log_assert(wire);
 
+					const LibertyAst *capacitance = node->find("capacitance");
+					if (capacitance)
+						wire->attributes[ID::capacitance] = capacitance->value;
+
 					if (dir && dir->value == "inout") {
 						wire->port_input = true;
 						wire->port_output = true;
@@ -672,8 +687,10 @@ struct LibertyFrontend : public Frontend {
 						continue;
 					}
 
-					if (dir && dir->value == "output")
+					if (dir && dir->value == "output") {
+						has_outputs = true;
 						wire->port_output = true;
+					}
 
 					if (flag_lib)
 						continue;
@@ -691,36 +708,35 @@ struct LibertyFrontend : public Frontend {
 								goto skip_cell;
 							}
 						}
+						simple_comb_cell = false;
 					} else {
 						RTLIL::SigSpec out_sig = parse_func_expr(module, func->value.c_str());
 						const LibertyAst *three_state = node->find("three_state");
 						if (three_state) {
 							out_sig = create_tristate(module, out_sig, three_state->value.c_str());
+							simple_comb_cell = false;
 						}
 						module->connect(RTLIL::SigSig(wire, out_sig));
 					}
+				}
 
-					if (flag_unit_delay) {
-						pool<Wire *> done;
+				if (node->id == "ff" || node->id == "ff_bank" ||
+						node->id == "latch" || node->id == "latch_bank" ||
+						node->id == "statetable")
+					simple_comb_cell = false;
+			}
 
-						for (auto timing : node->children)
-						if (timing->id == "timing" && timing->args.empty()) {
-							auto type = timing->find("timing_type");
-							auto related_pin = timing->find("related_pin");
-							if (!type || type->value != "combinational" || !related_pin)
-								continue;
-
-							Wire *related = module->wire(RTLIL::escape_id(related_pin->value));
-							if (!related)
-								log_error("Failed to find related pin %s for timing of pin %s on %s\n",
-										  related_pin->value.c_str(), log_id(wire), log_id(module));
-
-							if (done.count(related))
-								continue;
+			if (simple_comb_cell && has_outputs) {
+				module->set_bool_attribute(ID::abc9_box);
 
+				if (flag_unit_delay) {
+					for (auto wi : module->wires())
+					if (wi->port_input) {
+						for (auto wo : module->wires())
+						if (wo->port_output) {
 							RTLIL::Cell *spec = module->addCell(NEW_ID, ID($specify2));
-							spec->setParam(ID::SRC_WIDTH, 1);
-							spec->setParam(ID::DST_WIDTH, 1);
+							spec->setParam(ID::SRC_WIDTH, wi->width);
+							spec->setParam(ID::DST_WIDTH, wo->width);
 							spec->setParam(ID::T_FALL_MAX, 1000);
 							spec->setParam(ID::T_FALL_TYP, 1000);
 							spec->setParam(ID::T_FALL_MIN, 1000);
@@ -729,16 +745,29 @@ struct LibertyFrontend : public Frontend {
 							spec->setParam(ID::T_RISE_MIN, 1000);
 							spec->setParam(ID::SRC_DST_POL, false);
 							spec->setParam(ID::SRC_DST_PEN, false);
-							spec->setParam(ID::FULL, false);
+							spec->setParam(ID::FULL, true);
 							spec->setPort(ID::EN, Const(1, 1));
-							spec->setPort(ID::SRC, related);
-							spec->setPort(ID::DST, wire);
-							done.insert(related);
+							spec->setPort(ID::SRC, wi);
+							spec->setPort(ID::DST, wo);
 						}
 					}
 				}
 			}
 
+			if (design->has(cell_name)) {
+				Module *existing_mod = design->module(cell_name);
+				if (!flag_nooverwrite && !flag_overwrite && !existing_mod->get_bool_attribute(ID::blackbox)) {
+					log_error("Re-definition of cell/module %s!\n", log_id(cell_name));
+				} else if (flag_nooverwrite) {
+					log("Ignoring re-definition of module %s.\n", log_id(cell_name));
+					delete module;
+					goto skip_cell;
+				} else {
+					log("Replacing existing%s module %s.\n", existing_mod->get_bool_attribute(ID::blackbox) ? " blackbox" : "", log_id(cell_name));
+					design->remove(existing_mod);
+				}
+			}
+
 			module->fixup_ports();
 			design->add(module);
 			cell_count++;
