@@ -260,7 +260,7 @@ bool DriveChunkMultiple::try_append(DriveBitMultiple const &bit)
 		switch (single.type())
 		{
 			case DriveType::CONSTANT: {
-				single.constant().bits.push_back(constant);
+				single.constant().bits().push_back(constant);
 			} break;
 			case DriveType::WIRE: {
 				single.wire().width += 1;
@@ -295,8 +295,8 @@ bool DriveChunkMultiple::try_append(DriveChunkMultiple const &chunk)
 		switch (single.type())
 		{
 			case DriveType::CONSTANT: {
-				auto &bits = single.constant().bits;
-				bits.insert(bits.end(), constant.bits.begin(), constant.bits.end());
+				auto &bits = single.constant().bits();
+				bits.insert(bits.end(), constant.bits().begin(), constant.bits().end());
 			} break;
 			case DriveType::WIRE: {
 				single.wire().width += width;
@@ -349,7 +349,7 @@ bool DriveChunk::try_append(DriveBit const &bit)
 			none_ += 1;
 			return true;
 		case DriveType::CONSTANT:
-			constant_.bits.push_back(bit.constant());
+			constant_.bits().push_back(bit.constant());
 			return true;
 		case DriveType::WIRE:
 			return wire_.try_append(bit.wire());
@@ -375,7 +375,7 @@ bool DriveChunk::try_append(DriveChunk const &chunk)
 			none_ += chunk.none_;
 			return true;
 		case DriveType::CONSTANT:
-			constant_.bits.insert(constant_.bits.end(), chunk.constant_.bits.begin(), chunk.constant_.bits.end());
+			constant_.bits().insert(constant_.bits().end(), chunk.constant_.begin(), chunk.constant_.end());
 			return true;
 		case DriveType::WIRE:
 			return wire_.try_append(chunk.wire());
