@@ -1,5 +1,6 @@
 #include "kernel/yosys.h"
 #include "kernel/ff.h"
+#include "kernel/gzip.h"
 #include "libparse.h"
 #include <optional>
 
@@ -308,13 +309,12 @@ struct ClockgatePass : public Pass {
 		if (!liberty_files.empty()) {
 			LibertyMergedCells merged;
 			for (auto path : liberty_files) {
-				std::ifstream f;
-				f.open(path.c_str());
-				if (f.fail())
+				std::istream* f = uncompressed(path);
+				if (f->fail())
 					log_cmd_error("Can't open liberty file `%s': %s\n", path.c_str(), strerror(errno));
-				LibertyParser p(f);
+				LibertyParser p(*f, path);
 				merged.merge(p);
-				f.close();
+				delete f;
 			}
 			std::tie(pos_icg_desc, neg_icg_desc) =
 				find_icgs(merged.cells, dont_use_cells);
