@@ -90,12 +90,70 @@ namespace Yosys
 		bool eval(dict<std::string, bool>& values);
 	};
 
+	class LibertyInputStream {
+		std::istream &f;
+		std::vector<char> buffer;
+		size_t buf_pos = 0;
+		size_t buf_end = 0;
+		bool eof = false;
+
+		bool extend_buffer_once();
+		bool extend_buffer_at_least(size_t size = 1);
+
+		YS_COLD int get_cold();
+		YS_COLD int peek_cold(size_t offset);
+
+	public:
+		LibertyInputStream(std::istream &f) : f(f) {}
+
+		size_t buffered_size() { return buf_end - buf_pos; }
+		const char *buffered_data() { return buffer.data() + buf_pos; }
+
+		int get() {
+			if (buf_pos == buf_end)
+				return get_cold();
+			int c = buffer[buf_pos];
+			buf_pos += 1;
+			return c;
+		}
+
+		int peek(size_t offset = 0) {
+			if (buf_pos + offset >= buf_end)
+				return peek_cold(offset);
+			return buffer[buf_pos + offset];
+		}
+
+		void consume(size_t n = 1) {
+			buf_pos += n;
+		}
+
+		void unget() {
+			buf_pos -= 1;
+		}
+	};
+
+#ifndef FILTERLIB
+	class LibertyAstCache {
+		LibertyAstCache() {};
+		~LibertyAstCache() {};
+	public:
+		dict<std::string, std::shared_ptr<const LibertyAst>> cached;
+
+		bool cache_by_default = false;
+		dict<std::string, bool> cache_path;
+
+		std::shared_ptr<const LibertyAst> cached_ast(const std::string &fname);
+		void parsed_ast(const std::string &fname, const std::shared_ptr<const LibertyAst> &ast);
+		static LibertyAstCache instance;
+	};
+#endif
+
 	class LibertyMergedCells;
 	class LibertyParser
 	{
 		friend class LibertyMergedCells;
 	private:
-		std::istream &f;
+		LibertyInputStream f;
 		int line;
 
 		/* lexer return values:
@@ -110,15 +168,29 @@ namespace Yosys
 		void error(const std::string &str) const;
 
 	public:
-		const LibertyAst *ast;
+		std::shared_ptr<const LibertyAst> shared_ast;
+		const LibertyAst *ast = nullptr;
 
-		LibertyParser(std::istream &f) : f(f), line(1), ast(parse()) {}
-		~LibertyParser() { if (ast) delete ast; }
+		LibertyParser(std::istream &f) : f(f), line(1) {
+			shared_ast.reset(parse());
+			ast = shared_ast.get();
+		}
+
+#ifndef FILTERLIB
+		LibertyParser(std::istream &f, const std::string &fname) : f(f), line(1) {
+			shared_ast = LibertyAstCache::instance.cached_ast(fname);
+			if (!shared_ast) {
+				shared_ast.reset(parse());
+				LibertyAstCache::instance.parsed_ast(fname, shared_ast);
+			}
+			ast = shared_ast.get();
+		}
+#endif
 	};
 
 	class LibertyMergedCells
 	{
-		std::vector<const LibertyAst *> asts;
+		std::vector<std::shared_ptr<const LibertyAst>> asts;
 
 	public:
 		std::vector<const LibertyAst *> cells;
@@ -126,10 +198,7 @@ namespace Yosys
 		{
 			if (parser.ast) {
 				const LibertyAst *ast = parser.ast;
-				asts.push_back(ast);
-				// The parser no longer owns its top level ast, but we do.
-				// sketchy zone
-				parser.ast = nullptr;
+				asts.push_back(parser.shared_ast);
 				if (ast->id != "library")
 					parser.error("Top level entity isn't \"library\".\n");
 				for (const LibertyAst *cell : ast->children)
@@ -137,11 +206,6 @@ namespace Yosys
 						cells.push_back(cell);
 			}
 		}
-		~LibertyMergedCells()
-		{
-			for (auto ast : asts)
-				delete ast;
-		}
 	};
 
 }
